/*
Benchmark for Route Planning Algorithms
 
Copyright (c) Microsoft Corporation

All rights reserved. 

MIT License
 
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#pragma once

#include <iostream>
#include <vector>
#include <limits>

#include "heap.h"

namespace benchmark {

template<typename g_t>
class dijkstra {
public:

	// Expose some types.
	typedef g_t graph_t;
	typedef typename graph_t::vertex_id_t vertex_id_t;
	typedef typename graph_t::arc_id_t arc_id_t;
	typedef typename graph_t::weight_t weight_t;	
	
	// Constructor: Sets up data structures.
	dijkstra(const graph_t &g) : graph(g), inf(numeric_limits<weight_t>::max()), pqueue(g.num_vertices()), distances(g.num_vertices())
	{
	}
	
	
	// Runs a query from a certain source vertex s.
	inline void run_query(const vertex_id_t s) {
		assert(s >= 0 && s < graph.num_vertices());
		
		// Initialize data structures and reset distances.
		pqueue.clear();
		distances.assign(graph.num_vertices(), inf);
		
		// Add s to queue and set its distance to 0.
		distances[s] = 0;
		pqueue.update(s, 0);
		
		
		// Run the algorithm while queue is not empty.
		while (!pqueue.empty()) {
			// Extract next vertex from priority queue.
			vertex_id_t u(0);
			weight_t dist_su(0);
			pqueue.extract_min(u, dist_su);
			assert(u >= 0 && u < graph.num_vertices());
			assert(dist_su >= 0 && dist_su < inf);
						
			// Iterate all incident arcs.
			arc_id_t first_arc_id = graph.vertices[u].first_arc_id;
			arc_id_t end_arc_id = graph.vertices[u+1].first_arc_id;
			assert(first_arc_id >= 0 && first_arc_id < graph.num_arcs());
			assert(end_arc_id >= first_arc_id && end_arc_id <= graph.num_arcs());
			for (arc_id_t arc_id = first_arc_id; arc_id < end_arc_id; ++arc_id) {
				const vertex_id_t v = graph.arcs[arc_id].to_vertex_id;
				assert(v >= 0 && v < graph.num_vertices());
				
				// Skip arcs with weight infinity.
				if (graph.arcs[arc_id].weight >= inf) continue;
				
				// Compute distance of path s->u->v.
				const weight_t dist_suv = dist_su + graph.arcs[arc_id].weight;
				assert(dist_suv >= 0 && dist_suv < inf);
				
				// Skip this arc if it does not yield an improvement at v.
				if (dist_suv >= distances[v]) continue;
				
				// Update distance label at v, and update priority queue.
				distances[v] = dist_suv;
				pqueue.update(v, dist_suv);
			}
		}
	}
	
	
	// Returns the checksum of the last iteration.
	// Basically, sums up the distances to all vertices that do not have a label of infinity.
	inline unsigned int get_checksum() const {
		unsigned int checksum(0);
		for (size_t v = 0; v < graph.num_vertices(); ++v) {
			if (distances[v] == inf) continue;
			checksum += distances[v];
		}
		return checksum;
	}
	

protected:

	// This is the graph the algorithm is running on.
	const graph_t &graph;
	
	// This is an infinite weight.
	const weight_t inf;
	
	// This is a priority queue.
	heap<2, weight_t> pqueue;
	
	// These are the distance labels of the algorithm.
	vector<weight_t> distances;

};

}