/*
Benchmark for Route Planning Algorithms
 
Copyright (c) Microsoft Corporation

All rights reserved. 

MIT License
 
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#include <iostream>
#include <string>
#include <cstdlib>

using namespace std;

#include "version.h"
#include "graph.h"
#include "parser.h"
#include "dijkstra.h"
#include "random.h"
#include "tools.h"

int main(int argn, char **argv) {
	// Obtain command line arguments.
	string input_filename("data/usa-t.gr");
	if (argn > 1)
		input_filename = string(argv[1]);
	int num_queries(10);
	if (argn > 2)
		num_queries = min<int>(atoi(argv[2]), benchmark::num_random_numbers);
	int num_repetitions(9);
	if (argn > 3)
		num_repetitions = atoi(argv[3]);
		
		
	// Output choices of parameters.
	cout << "Call: " << argv[0] << " " << input_filename << " " << num_queries << " " << num_repetitions << endl
		<< "Version: " << benchmark::version_string << endl
		<< "Input: " << input_filename << endl
		<< "Queries: " << num_queries << endl
		<< "Repetitions: " << num_repetitions << endl
		<< endl;
		
	// Instantiate a default graph and parse it from the input file.
	benchmark::default_graph graph;
	if (!benchmark::parse_dimacs(input_filename, graph)) return -1;
	
	// Instantiate Dijkstra's algorithm.
	cout << endl << "Instantiating Dijkstra's algorithm... " << flush;
	benchmark::dijkstra<benchmark::default_graph> dij(graph);
	cout << "done." << endl;
	
	// Run queries.
	vector<double> running_times;
	for (int rep = 0; rep < num_repetitions; ++rep) {
		cout << endl << "[" << (rep+1) << "] Running " << num_queries << " " << (num_queries == 1 ? "query" : "queries") << " from " << (num_queries == 1 ? "vertex" : "vertices") << flush;
		const double timestamp_begin = benchmark::timestamp();
		unsigned int checksum(0);
		for (int q = 0; q < num_queries; ++q) {
			const benchmark::default_graph::vertex_id_t s = benchmark::random_numbers[q] % graph.num_vertices();
			cout << " " << s << flush;
			dij.run_query(s);
			checksum += dij.get_checksum();
		}
		const double elapsed_milliseconds = benchmark::timestamp()-timestamp_begin;
		running_times.push_back(elapsed_milliseconds);
		cout << " done (" << elapsed_milliseconds << " ms, checksum: 0x" << hex << checksum << dec << ")." << endl;
	}
	
	// Output median running time over all queries.
	const double median_running_time = benchmark::median(running_times);
	cout << endl << "Median running time over " << num_repetitions << " " << (num_repetitions == 1 ? "test" : "tests") << " was: " <<  median_running_time << "ms" << endl;
	
	return 0;
}